@@ -17,7 +17,7 @@ from trytond.wizard import (
     Button, StateAction, StateTransition, StateView, Wizard)
 
 
-_ZERO = Decimal('0.0')
+_ZERO = Decimal('0')
 
 
 class Batch(Workflow, ModelSQL, ModelView):
@@ -882,12 +882,15 @@ class BatchLine(ModelSQL, ModelView):
                 elif side == 'debit':
                     tax_debit, tax_credit = _ZERO, tax_amount
                     tax_account = tax_debit_account
+                # Change sides for compensatory reverse charge taxes
+                # to match upstream behavior
+                if reverse_charge:
+                    if tax.rate and tax.rate < _ZERO:
+                        tax_debit, tax_credit = -tax_credit, -tax_debit
 
-                tax_base_amount = base_amount
                 tax_tax_amount = tax_amount
                 if cancel:
                     tax_debit, tax_credit = -tax_debit, -tax_credit
-                    tax_base_amount = -tax_base_amount
                     tax_tax_amount = -tax_tax_amount
 
                 # Create the move line for the tax(es)
@@ -897,13 +900,16 @@ class BatchLine(ModelSQL, ModelView):
 
                 if tax_group_kind == 'purchase':
                     tax_tax_amount = -tax_tax_amount
-                    tax_base_amount = -tax_base_amount
 
                 # Create the tax_line(s) for the tax(es)
                 tax_line = self._get_tax_line(tax_tax_amount, 'tax', tax)
                 tax_move_line.tax_lines += (tax_line,)
 
                 # Create the tax_line(s) for the base
+                # The base amount has always the same sign as the tax amount
+                tax_base_amount = abs(base_amount)
+                if tax_tax_amount < _ZERO:
+                    tax_base_amount = -tax_base_amount
                 base_tax_line = self._get_tax_line(tax_base_amount,
                     'base', tax)
                 base_tax_lines.append(base_tax_line)
@@ -956,7 +962,7 @@ class BatchLine(ModelSQL, ModelView):
         MoveLine = pool.get('account.move.line')
 
         return MoveLine(
-            description=self.posting_text,
+            description=self.posting_text or account.name,
             debit=debit,
             credit=credit,
             account=account,
@@ -966,6 +972,7 @@ class BatchLine(ModelSQL, ModelView):
             maturity_date=self.maturity_date,
             code=self.sequence,
             reference=self.reference or '',
+            origin=self,
             tax_lines=[],
             )
 
